@@ -10,14 +10,16 @@
 #' obtained from `anova(model)` which might not always be appropriate. See
 #' details.
 #'
-#' @param model A model, ANOVA object, or the result of `parameters::model_parameters`.
+#' @param model An ANOVA table (or an ANOVA-like table, e.g., outputs from
+#'   `parameters::model_parameters`), or a statistical model for which such a
+#'   table can be extracted. See details.
 #' @param partial If `TRUE`, return partial indices.
-#' @param generalized If TRUE, returns generalized Eta Squared, assuming all
-#'   variables are manipulated. Can also be a character vector of observed
-#'   (non-manipulated) variables, in which case generalized Eta Squared is
-#'   calculated taking these observed variables into account. For `afex_aov`
-#'   model, when `generalized = TRUE`, the observed variables are extracted
-#'   automatically from the fitted model, if they were provided then.
+#' @param generalized A character vector of observed (non-manipulated) variables
+#'   to be used in the estimation of a generalized Eta Squared. Can also be
+#'   `TRUE`, in which case generalized Eta Squared is estimated assuming *none*
+#'   of the variables are observed (all are manipulated). (For `afex_aov`
+#'   models, when `TRUE`, the observed variables are extracted automatically
+#'   from the fitted model, if they were provided during fitting.
 #' @param verbose Toggle warnings and messages on or off.
 #' @inheritParams chisq_to_phi
 #' @param ... Arguments passed to or from other methods.
@@ -35,25 +37,30 @@
 #'
 #' @details
 #'
-#' For `aov`, `aovlist` and `afex_aov` models, and for `anova` objects that
-#' provide Sums-of-Squares, the effect sizes are computed directly using
-#' Sums-of-Squares (for `mlm` / `maov` models, effect sizes are computed for
-#' each response separately). For all other model, effect sizes are approximated
-#' via test statistic conversion of the omnibus *F* statistic provided by the
-#' appropriate `anova()` method (see [`F_to_eta2()`] for more details.)
+#' For `aov` (or `lm`), `aovlist` and `afex_aov` models, and for `anova` objects
+#' that provide Sums-of-Squares, the effect sizes are computed directly using
+#' Sums-of-Squares. (For `maov` (or `mlm`) models, effect sizes are computed for
+#' each response separately.)
+#' \cr\cr
+#' For other ANOVA tables and models (converted to ANOVA-like tables via
+#' `anova()` methods), effect sizes are approximated via test statistic
+#' conversion of the omnibus *F* statistic provided by the (see [`F_to_eta2()`]
+#' for more details.)
 #'
 #' ## Type of Sums of Squares
-#' The sums of squares (or *F* statistics) used for the computation of the
-#' effect sizes is based on those returned by `anova(model)` (whatever those may
-#' be - for `aov` and `aovlist` these are *type-1* sums of squares; for
-#' `lmerMod` (and `lmerModLmerTest`) these are *type-3* sums of squares). Make
-#' sure these are the sums of squares you are interested in; You might want to
-#' pass the result of `car::Anova(mode, type = 2)` or `type = 3` instead of the
-#' model itself, or use the `afex` package to fit ANOVA models.
+#' When `model` is a statistical model, the sums of squares (or *F* statistics)
+#' used for the computation of the effect sizes are based on those returned by
+#' `anova(model)`. Different models have different default output type. For
+#' example, for `aov` and `aovlist` these are *type-1* sums of squares, but for
+#' `lmerMod` (and `lmerModLmerTest`) these are *type-3* sums of squares. Make
+#' sure these are the sums of squares you are interested in. You might want to
+#' convert your model to an ANOVA(-like) table yourself and then pass the result
+#' to `eta_squared()`. See examples below for use of `car::Anova()` and the
+#' `afex` package.
 #' \cr\cr
 #' For type 3 sum of squares, it is generally recommended to fit models with
-#' *`contr.sum` factor weights* and *centered covariates*, for sensible results.
-#' See examples and the `afex` package.
+#' *orthogonal factor weights* (e.g., `contr.sum`) and *centered covariates*,
+#' for sensible results. See examples and the `afex` package.
 #'
 #' ## Un-Biased Estimate of Eta
 #' Both ***Omega*** and ***Epsilon*** are unbiased estimators of the
@@ -87,6 +94,7 @@
 #'
 #' @inheritSection effectsize_CIs Confidence (Compatibility) Intervals (CIs)
 #' @inheritSection effectsize_CIs CIs and Significance Tests
+#' @inheritSection print.effectsize_table Plotting with `see`
 #'
 #' @seealso [F_to_eta2()]
 #' @family effect sizes for ANOVAs
@@ -249,11 +257,15 @@ epsilon_squared <- function(model,
 
 #' @rdname eta_squared
 #' @inheritParams F_to_f
+#' @param method What effect size should be used as the basis for Cohen's *f*?
 #' @param model2 Optional second model for Cohen's f (/squared). If specified,
 #'   returns the effect size for R-squared-change between the two models.
 #' @export
 cohens_f <- function(model,
-                     partial = TRUE, squared = FALSE, model2 = NULL,
+                     partial = TRUE, generalized = FALSE,
+                     squared = FALSE,
+                     method = c("eta", "omega", "epsilon"),
+                     model2 = NULL,
                      ci = 0.95, alternative = "greater",
                      verbose = TRUE, ...) {
   alternative <- .match.alt(alternative, FALSE)
@@ -265,19 +277,27 @@ cohens_f <- function(model,
     ))
   }
 
-  res <- eta_squared(model,
+  method <- match.arg(method)
+  f <- switch(method,
+    eta = eta_squared,
+    generalized = generalized,
+    omega = omega_squared,
+    epsilon = epsilon_squared
+  )
+
+  res <- f(model,
     partial = partial,
     ci = ci, alternative = alternative,
     verbose = verbose,
     ...
   )
 
-  if ("Eta2_partial" %in% colnames(res)) {
-    res$Eta2_partial <- res$Eta2_partial / (1 - res$Eta2_partial)
-    colnames(res)[colnames(res) == "Eta2_partial"] <- "Cohens_f2_partial"
+  es_name <- get_effectsize_name(colnames(res))
+  res[[es_name]] <- res[[es_name]] / (1 - res[[es_name]])
+  if (grepl("_partial", es_name)) {
+    colnames(res)[colnames(res) == es_name] <- "Cohens_f2_partial"
   } else {
-    res$Eta2 <- res$Eta2 / (1 - res$Eta2)
-    colnames(res)[colnames(res) == "Eta2"] <- "Cohens_f2"
+    colnames(res)[colnames(res) == es_name] <- "Cohens_f2"
   }
 
   if (!is.null(ci)) {
@@ -296,18 +316,25 @@ cohens_f <- function(model,
   if ("CI" %in% colnames(res)) attr(res, "ci_method") <- list(method = "ncp", distribution = "F")
   class(res) <- unique(c("effectsize_anova", "effectsize_table", "see_effectsize_table", class(res)))
   attr(res, "approximate") <- isTRUE(attr(res, "approximate", exact = TRUE))
+  attr(res, "table_footer") <- if (method != "eta") sprintf("Based on %s squared.", paste0(toupper(substring(method, 1, 1)), substring(method, 2)))
   res
 }
 
 #' @rdname eta_squared
 #' @export
 cohens_f_squared <- function(model,
-                             partial = TRUE, squared = TRUE, model2 = NULL,
+                             partial = TRUE, generalized = FALSE,
+                             squared = TRUE,
+                             method = c("eta", "omega", "epsilon"),
+                             model2 = NULL,
                              ci = 0.95, alternative = "greater",
                              verbose = TRUE, ...) {
   cohens_f(
     model,
-    partial = partial, squared = squared, model2 = model2,
+    partial = partial, generalized = generalized,
+    squared = squared,
+    method = method,
+    model2 = model2,
     ci = ci, alternative = alternative,
     verbose = verbose, ...
   )
@@ -315,7 +342,6 @@ cohens_f_squared <- function(model,
 
 
 #' @keywords internal
-#' @importFrom insight model_info
 .cohens_f_delta <- function(model, model2,
                             squared = FALSE,
                             ci = 0.95, alternative = "greater",
@@ -329,7 +355,7 @@ cohens_f_squared <- function(model,
   }
 
   # Anova
-  ANOVA <- anova(model, model2)
+  ANOVA <- stats::anova(model, model2)
   out <- F_to_f(ANOVA[2, "F"], abs(ANOVA[2, "Df"]), min(ANOVA["Res.Df"]),
     ci = ci, alternative = alternative,
     squared = squared
@@ -736,24 +762,83 @@ cohens_f_squared <- function(model,
     UseMethod(".anova_es")
   }
 
+
 #' @keywords internal
 #' @importFrom stats anova
-.anova_es.default <- function(model,
-                              type = c("eta", "omega", "epsilon"),
-                              partial = TRUE,
-                              generalized = FALSE,
-                              ci = 0.95, alternative = "greater",
-                              verbose = TRUE,
-                              ...) {
-  .anova_es.anova(
-    stats::anova(model),
-    type = type,
-    partial = partial,
-    generalized = generalized,
-    ci = ci,
-    alternative = alternative,
-    verbose = verbose
+.anova_es.default <- function(model, ...) {
+  .anova_es.anova(stats::anova(model), ...)
+}
+
+#' @keywords internal
+.anova_es.anova <- function(model,
+                            type = c("eta", "omega", "epsilon"),
+                            partial = TRUE,
+                            generalized = FALSE,
+                            ci = 0.95, alternative = "greater",
+                            verbose = TRUE,
+                            include_intercept = FALSE,
+                            ...) {
+  F.nm <- c("F value", "approx F", "F-value", "F")
+  df.nm <- c("NumDF", "num Df", "numDF", "npar", "Df")
+  df_error.nm <- c("DenDF", "den Df", "denDF", "df_error", "Df.res")
+
+  # If there is no df_error *or* if there IS a residuals row...
+  if (!any(df_error.nm %in% colnames(model))) {
+    # Pass to AOV method
+    res <- .anova_es.aov(model,
+      partial = partial,
+      type = type,
+      generalized = generalized,
+      ci = ci, alternative = alternative,
+      verbose = verbose,
+      include_intercept = include_intercept,
+      ...
+    )
+    return(res)
+  }
+
+  if (!any(F.nm %in% colnames(model)) || !any(df.nm %in% colnames(model))) {
+    insight::format_error(
+      "ANOVA table does not have F values or degrees of freedom,",
+      "cannot compute effect size."
+    )
+  }
+
+  Fi <- F.nm[F.nm %in% colnames(model)]
+  dfi <- df.nm[df.nm %in% colnames(model)]
+  df_errori <- df_error.nm[df_error.nm %in% colnames(model)]
+
+  if (length(dfi) > 1L) {
+    dfi <- dfi[1] # For MANOVA this should not use the MV-df
+  }
+
+  # Clean up table ---
+  par_table <- data.frame(
+    Parameter = rownames(model),
+    F = model[, Fi],
+    df = model[, dfi],
+    df_error = model[, df_errori],
+    stringsAsFactors = FALSE
   )
+  par_table <- par_table[!par_table[["Parameter"]] %in% "Residuals", ]
+
+  out <-
+    .es_aov_table(
+      par_table,
+      type = type,
+      partial = partial,
+      generalized = generalized,
+      ci = ci,
+      alternative = alternative,
+      verbose = verbose,
+      include_intercept = include_intercept
+    )
+
+  attr(out, "anova_type") <- tryCatch(attr(parameters::model_parameters(model, verbose = FALSE, effects = "fixed"), "anova_type"),
+    error = function(...) 1
+  )
+  attr(out, "approximate") <- TRUE
+  out
 }
 
 #' @keywords internal
@@ -766,31 +851,38 @@ cohens_f_squared <- function(model,
                           ci = 0.95, alternative = "greater",
                           verbose = TRUE,
                           ...) {
-  if (!inherits(model, c("Gam", "anova"))) {
-    # Pass to ANOVA table method
-    res <- .anova_es.anova(
-      stats::anova(model),
-      type = type,
-      partial = partial,
-      generalized = generalized,
-      ci = ci, alternative = alternative,
-      verbose = verbose,
-      ...
-    )
-    return(res)
-  }
+  # if (!inherits(model, c("Gam", "anova"))) {
+  #   # Pass to ANOVA table method
+  #   res <- .anova_es.anova(
+  #     stats::anova(model),
+  #     type = type,
+  #     partial = partial,
+  #     generalized = generalized,
+  #     ci = ci, alternative = alternative,
+  #     verbose = verbose,
+  #     ...
+  #   )
+  #   return(res)
+  # }
 
+  # TODO this should be in .anova_es.anvoa
+  # TODO the aoc method should convert to an anova table, then pass to anova
   params <- parameters::model_parameters(model, verbose = verbose, effects = "fixed")
-  out <- .es_aov_simple(as.data.frame(params), type, partial, generalized, ci, alternative, verbose = verbose, ...)
+  out <- .es_aov_simple(as.data.frame(params),
+    type = type,
+    partial = partial, generalized = generalized,
+    ci = ci, alternative = alternative, verbose = verbose, ...
+  )
   if (is.null(attr(out, "anova_type"))) attr(out, "anova_type") <- attr(params, "anova_type")
   out
 }
 
-.anova_es.lm <- .anova_es.aov
+#' @keywords internal
+.anova_es.lm <- function(model, ...) {
+  .anova_es.aov(stats::aov(model), ...)
+}
 
-.anova_es.glm <- .anova_es.aov
-
-.anova_es.manova <- .anova_es.aov
+.anova_es.glm <- .anova_es.lm
 
 #' @keywords internal
 #' @importFrom parameters model_parameters
@@ -824,76 +916,6 @@ cohens_f_squared <- function(model,
   out
 }
 
-#' @keywords internal
-.anova_es.anova <- function(model,
-                            type = c("eta", "omega", "epsilon"),
-                            partial = TRUE,
-                            generalized = FALSE,
-                            ci = 0.95, alternative = "greater",
-                            verbose = TRUE,
-                            include_intercept = FALSE,
-                            ...) {
-  F.nm <- c("F value", "approx F", "F-value", "F")
-  df.nm <- c("NumDF", "num Df", "numDF", "npar", "Df")
-  df_error.nm <- c("DenDF", "den Df", "denDF", "df_error", "Df.res")
-
-  # If there is no df_error *or* is there IS a residuals row...
-  if (!any(df_error.nm %in% colnames(model))) {
-    # Pass to AOV method
-    res <- .anova_es.aov(model,
-      partial = partial,
-      type = type,
-      generalized = generalized,
-      ci = ci, alternative = alternative,
-      verbose = verbose,
-      include_intercept = include_intercept,
-      ...
-    )
-    return(res)
-  }
-
-  if (!any(F.nm %in% colnames(model)) || !any(df.nm %in% colnames(model))) {
-    insight::format_error("ANOVA table does not have F values or degrees of freedom - cannot compute effect size.")
-  }
-
-  Fi <- F.nm[F.nm %in% colnames(model)]
-  dfi <- df.nm[df.nm %in% colnames(model)]
-  df_errori <- df_error.nm[df_error.nm %in% colnames(model)]
-
-  if (length(dfi) > 1L) {
-    dfi <- dfi[1] # For MANOVA this should not use the MV-df
-  }
-
-  # Clean up table ---
-  par_table <- data.frame(
-    Parameter = rownames(model),
-    F = model[, Fi],
-    df = model[, dfi],
-    df_error = model[, df_errori]
-  )
-  par_table <- par_table[!par_table[["Parameter"]] %in% "Residuals", ]
-
-  out <-
-    .es_aov_table(
-      par_table,
-      type = type,
-      partial = partial,
-      generalized = generalized,
-      ci = ci,
-      alternative = alternative,
-      verbose = verbose,
-      include_intercept = include_intercept
-    )
-
-  attr(out, "anova_type") <- tryCatch(attr(parameters::model_parameters(model, verbose = FALSE, effects = "fixed"), "anova_type"),
-    error = function(...) 1
-  )
-  attr(out, "approximate") <- TRUE
-  out
-}
-
-
-
 
 # Utils -------------------------------------------------------------------
 
